# -*- coding: utf-8 -*-
"""
    bracket
    ~~~~~

    Bracket is an Elegant static site generator.

    :copyright: (c) 2020 by Ceorleorn(https://github.com/ceorleorn).
    :license: MIT License, see LICENSE for more details.
"""
from .context import PagesContext


# All Context
pages = PagesContext
resources = None



# -*- coding: utf-8 -*-
"""
    bracket
    ~~~~~

    Bracket is an Elegant static site generator.

    :copyright: (c) 2020 by Ceorleorn(https://github.com/ceorleorn).
    :license: MIT License, see LICENSE for more details.
"""
from jinja2 import Template
from .pages import getSimplePages
from .info import __version__


class PagesContext(object):
    def __init__(self, objdict):
        try:
            objdict["title"]
            objdict["content"]
        except:
            raise ValueError("The return value is missing a required key.")

        if not isinstance(objdict["title"], str):
            raise TypeError("The title must be a string.")

        if not isinstance(objdict["content"], Template):
            raise TypeError("The content must be a jinja2.Template.")

        self.objdict = self.checkUIObjDict(objdict)

        self.title = objdict["title"]
        self.content = objdict["content"]
        self.headermeta = objdict["ui"]["headermeta"]
        self.lang = objdict["ui"]["lang"]

    def checkUIObjDict(self, objdict):
        checkitems = {
            "headermeta": "",
            "lang": "zh-CN",
            "description": "A simple pages",
            "keywords": "bracket,web,page",
            "author": "user",
        }

        try:
            objdict["ui"]
        except:
            objdict["ui"] = {}

        for itemsname, itemsvalue in checkitems.items():
            try:
                objdict["ui"][itemsname]
            except:
                objdict["ui"][itemsname] = itemsvalue

        return objdict

    def generateContext(self):
        try:
            res = self.objdict["resources"]
        except:
            res = {}

        context = {
            "bracket": RenderContext(
                {"version": str(__version__), "title": self.objdict["title"]}
            ),
            "i18n": None,
        }

        for obj, value in res.items():
            i = False
            try:
                context[obj]
                i = True
            except:
                i = False
            if i:
                raise NameError("Resource name is same.")
            context[obj] = value

        return context

    def render(self, template=getSimplePages):
        dispatch = self.content.render(self.generateContext())

        objdict = {
            "title": self.title,
            "headermeta": self.headermeta,
            "lang": self.lang,
            "content": dispatch,
            "bracketVersion": __version__,
        }

        return str(template(objdict))


class RenderContext(object):
    def __init__(self, attribute={}):
        for attr, value in attribute.items():
            setattr(self, attr, value)

    def res(self, url: str, file_type=None):
        if not url.startswith("/"):
            raise NameError("The resource name must start with a forward slash.")

        return "/static" + url

    def url(self, url):
        return 0


class StaticContext(object):
    def __init__(self, install_resources):
        self.install_resources = install_resources

    def install(self, name, resources):
        if not self.isInstalled(name):
            self.add(name, resources)
        else:
            raise NameError("Resource name is same.")

    def add(self, name, resources):
        self.install_resources[name] = resources

    def isInstalled(self, name):
        i = False
        try:
            self.install_resources[name]
            i = True
        except:
            i = False
        return i



# -*- coding: utf-8 -*-
"""
    bracket
    ~~~~~

    Bracket is an Elegant static site generator.

    :copyright: (c) 2020 by Ceorleorn(https://github.com/ceorleorn).
    :license: MIT License, see LICENSE for more details.
"""
from .context import PagesContext, StaticContext
from .configs import Configs, default_configs
from .debug import serve as debug
import importlib
import os


class WebSite(object):
    def __init__(self, import_name, import_modules={}):
        """
        Main application objects of bracket.

        ===================================
        from bracket import WebSite

        app = WebSite(__name__)
        ===================================

        """
        self.import_name = import_name
        self.import_modules = import_modules

        self.pages_list = []
        self.resources_list = []
        self.extensions_list = []

        self.config = Configs()
        self.config_default = default_configs

    def add_page(self, rule, viewfunc):
        """
        Add the underlying interface of interface view function.

        ===================================
        from bracket import WebSite
        from jinja2 import Template

        app = WebSite(__name__)

        def hello(context):
            return context({
                "title":"Welcome!!!",
                "content":Template('''<h1>Hello</h1>'''),
                "resources":{}
            })

        app.add_page("/",hello)
        ===================================

        """
        if not rule.startswith("/"):
            raise NameError("The route must start with a forward slash.")

        self.pages_list.append({"rule": rule, "viewfunc": viewfunc})

    def add_resource(self, func):
        """
        Add the underlying interface of static resources.

        ===================================
        from bracket import WebSite
        from jinja2 import Template

        app = WebSite(__name__)

        def hello(static):
            return static([])

        app.add_resources("/",hello)
        ===================================

        """
        self.resources_list.append(func)

    def pages(self, rule):
        """
        Add interface view function.

        ===================================
        from bracket import WebSite
        from jinja2 import Template

        app = WebSite(__name__)

        @app.pages("/")
        def hello(context):
            return context({
                "title":"Welcome!!!",
                "content":Template('''<h1>Hello</h1>'''),
                "resources":{}
            })

        ===================================

        """

        def decorator(f):
            self.add_page(rule=rule, viewfunc=f)
            return f

        return decorator

    def resources(self):
        """
        Add interface static resources.

        ===================================
        from bracket import WebSite
        from jinja2 import Template

        app = WebSite(__name__)

        @app.resources("/")
        def hello(static):
            return static([])

        ===================================

        """

        def decorator(f):
            self.add_resource(func=f)
            return f

        return decorator

    def dispatch(self, route):
        """
        Render the interface view function that has been added.

        ===================================
        from bracket import WebSite
        from jinja2 import Template

        app = WebSite(__name__)

        @app.pages("/")
        def hello(context):
            return context({
                "title":"Welcome!!!",
                "content":Template('''<h1>Hello</h1>'''),
                "resources":{}
            })

        app.dispatch("/")

        ===================================

        They return:

        ===================================

        <!DOCTYPE html>
        <html lang="zh-CN">
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <meta name="generator" content="Bracket (0.0.1) & Jinja2 ">

            <title>Welcome!!!</title>
        </head>
        <body>
            <div id="bracketapp">
                <h1>Hello</h1>
            </div>
        </body>
        </html>

        ===================================

        """
        dispatchs = None

        for obj in self.pages_list:
            if obj["rule"] == route:
                dispatch = obj["viewfunc"](PagesContext)
                if isinstance(dispatch, PagesContext):
                    dispatchs = dispatch.render()
                else:
                    dispatchs = dispatch

        return dispatchs

    def serve(self, serveconfig=None):
        """
        Use Flask to preview the current web site.

        $ pip install flask

        ===================================
        from bracket import WebSite
        from jinja2 import Template

        app = WebSite(__name__)

        @app.pages("/")
        def hello(context):
            return context({
                "title":"Welcome!!!",
                "content":Template('''<h1>Hello</h1>'''),
                "resources":{}
            })

        app.serve()

        ===================================

        They output and running on http://localhost:5000/

        ===================================

        Running on http://localhost:5000
          * Serving Flask app "Bracket" (lazy loading)
          * Environment: production
            WARNING: This is a development server. Do not use it in a production deployment.
            Use a production WSGI server instead.
          * Debug mode: off
          * Running on http://localhost:5000/ (Press CTRL+C to quit)

        ===================================

        """
        if not serveconfig:
            serveconfig = {
                "host": self.config.get("SERVE_HOST"),
                "port": self.config.get("SERVE_PORT"),
                "debug": self.config.get("SERVE_DEBUG"),
            }
        print("Running on http://localhost:" + str(serveconfig["port"]))
        debug(self, serveconfig)

    def escape(self, url: str):
        """
        Convert route to file path.

        ===================================
        from bracket import WebSite

        app = WebSite(__name__)

        app.escape("/")
        >>> "/index.html"

        app.escape("/hello")
        >>> "/hello.html"

        ===================================

        """
        if not url.startswith("/"):
            raise NameError("The url must start with a forward slash.")

        if url == "/":
            url = "/index"

        return str(url + ".html")

    def build(self):
        """
        Render the interface view function that has been added.

        ===================================
        from bracket import WebSite
        from jinja2 import Template

        app = WebSite(__name__)

        @app.pages("/")
        def hello(context):
            return context({
                "title":"Welcome!!!",
                "content":Template('''<h1>Hello</h1>'''),
                "resources":{}
            })

        app.build()

        ===================================

        They return and saved file:

        ===================================

        Building Staticfile with you ...

        Every thing was Ok.

        ===================================s

        gitpod /workspace/bracket/docs $ ls
        >>>index.html

        ===================================

        """
        print("Building Staticfile with you ...\n")

        try:
            os.mkdir(self.config.get("BUILD_OUTPUTDIR"))
        except:
            pass

        for view in self.pages_list:
            static = self.dispatch(view["rule"])
            if static == None:
                continue

            filepath = self.config.get("BUILD_OUTPUTDIR") + self.escape(view["rule"])

            try:
                os.mkdir(os.path.dirname(filepath))
            except:
                pass
            
            with open(filepath, "w+", encoding="utf-8") as file:
                file.write(str(static))

            print("Every thing was Ok.")

    def loader_extension(self, extension):
        """
        Load extension object.

        ===================================
        from bracket import WebSite

        class Extension(object):
            def __init__(self,configs):
                self.configs = configs
                self.moduels = {"createLink": self.createLink}

            def createLink(self,text,link):
                return Template('''<a href="{{ link }}">{{ text }}</a>''').render(text=text,link=link)

        app = WebSite(__name__)

        app.loader_extension(Extension)
        ===================================

        """
        if not isinstance(extension, object):
            raise TypeError("Extension must be a object.")

        if not hasattr(extension, "modules"):
            raise TypeError("Missing attribute: modules.")

        if not isinstance(extension.moduels, dict):
            raise TypeError("Modules must be a dict.")

        self.extensions_list.append(extension)

    def loader_resources(self):
        """
        Load all static resources that have been added

        ===================================
        from bracket import WebSite

        app = WebSite(__name__)

        @app.resources("/")
        def hello(static):
            return static([
                {"filename":"link.txt","content":"bracket.ink"}
            ])

        app.loader_resources()
        ===================================

        """
        resources = []
        for loaderfunc in self.resources_list:
            dispatch = loaderfunc(StaticContext)
            for resource in dispatch:
                resources.append(resource)
        return resources



# -*- coding: utf-8 -*-
"""
    bracket
    ~~~~~

    Bracket is an Elegant static site generator.

    :copyright: (c) 2020 by Ceorleorn(https://github.com/ceorleorn).
    :license: MIT License, see LICENSE for more details.
"""

from jinja2 import Template

simpleTemplate = """<!DOCTYPE html>
<html lang="{{ objdict['lang'] }}">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="generator" content="Bracket ({{ objdict['bracketVersion'] }}) & Jinja2 ">
    {{ objdict['headermeta'] }}
    <title>{{ objdict['title'] }}</title>
</head>
<body>
    <div id="bracketapp">
        {{ objdict['content'] }}
    </div>
</body>
</html>"""


def getSimplePages(objdict):
    dispatch = Template(simpleTemplate).render({"objdict": objdict})
    return dispatch



# -*- coding: utf-8 -*-
"""
    bracket
    ~~~~~

    Bracket is an Elegant static site generator.

    :copyright: (c) 2020 by Ceorleorn(https://github.com/ceorleorn).
    :license: MIT License, see LICENSE for more details.
"""

default_configs = {
    "BUILD_OUTPUTDIR": "./docs",
    "BUILD_DEBUG": False,
    "I18N_OPEN": False,
    "I18N_DATA": {"zh-CN"},
    "SERVE_HOST": "localhost",
    "SERVE_PORT": 5000,
    "SERVE_DEBUG": False,
}


class Configs(object):
    def __init__(self):
        self.configs = default_configs

    def setconfig(self, name, value):
        self.configs[name] = value

    def delete(self, name):
        del self.configs[name]

    def get(self, name):
        try:
            return self.configs[name]
        except:
            return None



# -*- coding: utf-8 -*-
"""
    bracket
    ~~~~~

    Bracket is an Elegant static site generator.

    :copyright: (c) 2020 by Ceorleorn(https://github.com/ceorleorn).
    :license: MIT License, see LICENSE for more details.
"""
import importlib


def serve(app, serveconfig):
    try:
        flask = importlib.import_module("flask")
    except:
        raise RuntimeError("Flask is not installed.")

    app = flask.Flask("Bracket", static_url_path="/static")

    @app.route("/", methods=["GET", "POST"])
    def indexhandler():
        dispatch = app.dispatch("/")
        if not (dispatch == None):
            return dispatch
        else:
            return flask.abort(404)

    @app.route("/<url>", methods=["GET", "POST"])
    def otherurlhandler(url):
        dispatch = app.dispatch("/" + url)
        if not (dispatch == None):
            return dispatch
        else:
            return flask.abort(404)

    app.run(
        host=serveconfig["host"], port=serveconfig["port"], debug=serveconfig["debug"]
    )



import setuptools
import re

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("bracket/info.py", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

setuptools.setup(
    name="bracket",
    version=version,
    packages=setuptools.find_packages(),
    author="Ceorleorn",
    author_email="snbck@qq.com",
    description="Bracket is an Elegant static site generator.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bracketing/bracket/",
    install_requires=["Jinja2>=2.10.1"],
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding: utf-8 -*-
"""
    bracket
    ~~~~~

    Bracket is an Elegant static site generator.

    :copyright: (c) 2020 by Ceorleorn(https://github.com/ceorleorn).
    :license: MIT License, see LICENSE for more details.
"""

from .app import WebSite
from .context import PagesContext
from .pages import getSimplePages

from .globals import pages
from .globals import resources

from .info import __version__


__all__ = ["WebSite", "PagesContext", "getSimplePages" "__version__"]
